// slist.c -- lists of strings in the format of an args list, 0 terminated.


#include <malloc.h>
#include <stdio.h>
#include <fcntl.h> // popen()
//#define __USE_SVID 1
//#define __USE_BSD 1
//#define __USE_XOPEN_EXTENDED 1
//#define __USE_XOPEN2K8 1
#include <string.h>


#include <unistd.h>
#include <stdio.h>

#include "runc.h"


// returns an initialized pointer to a list base
char** slist_new() { 
  void** p = objlist_new();
  return (char**)p;
}

// returns string at index
char * slist_get(char*** plist, int index)
{
  if(index >= slist_count(plist))
    return 0;
  if(index < 0)
    return 0;
  return (*plist)[index];
}


// removes items and deletes the base
void slist_delete(char*** plist) {
  int n = slist_count(plist);
  for(int i = 0; i < n; i++)
    free(slist_get(plist, i));
  objlist_delete((OBJLIST*)plist);
}

// returns number of items in the list
//int slist_count(char** list) { return objlist_count((OBJLIST)list);}

// returns number of items in the list
// int slist_getCount(char*** plist) { return objlist_getCount((OBJLIST*)plist);}
int slist_count(char*** plist) { return objlist_count((OBJLIST*)plist);}

// removes items, does not resize or delete the base
void slist_clear(char*** plist) 
{
  int n = slist_count(plist);
  for(int i = 0; i < n; i++)
    free(slist_get(plist, i));
  objlist_clear((OBJLIST*)plist);
}

// adds a new items at the end of the list
void slist_append(char*** plist, const char* s)
{
  char* str = strdup(s);
  objlist_append((OBJLIST*)plist, (OBJLIST_ITEM*)str);
}

// removes (and frees) item at index. Does not resize the list
void slist_remove(char*** plist, int index) 
{
  free(slist_get(plist, index));
  objlist_remove((OBJLIST*)plist, index);
}


// adds list list2 to list1, does not delete list that's added.
// plist2 can be any standard null terminated arg list or slist.
void slist_addList(char*** plist1, char*** plist2)
{
  char** s = *plist2;
  while(*s)
    slist_append(plist1, *s++);
}

// inserts new item at index
void slist_insert(char*** plist,  int index, const char* s)
{
  // This is causing some valgrind errors somehow, and it shows up in
  // objlist in a couple of places: objlist_insert(), and objlist_append()
  // as an invalid write.
  const char* str = strdup(s);
  objlist_insert((OBJLIST*)plist, index, (OBJLIST_ITEM*)str);
}

// remove one trailing newline from a string
static void trim_newline(char* buf)
{
  char* p = buf + strlen(buf) -1;
  if(*p == '\n')
    *p-- = 0;
  if(*p == '\r')    
    *p = 0;
}

// Loads list with strings from a piped command,
// returns 0 on success
int slist_pipeRead(char*** plist, const char* pipecmd)
{
  int intNu = 0;
  intNu = intNu;
  size_t bufsize = 64000;
  char* buf = (char*) malloc(bufsize);
  char** list = *plist;
  FILE* fp = popen(pipecmd, "r");
  if(! fp) return 1;
  while(! feof(fp))
  {
    *buf = 0;
    intNu = getline(&buf, &bufsize, fp);
    
    // done if nothing is read
    if(! *buf)
      break;
    
    trim_newline(buf);
    slist_append(&list, buf);
    usleep(100);
  }
  fclose(fp);
  
  // save the result and return ok
  free(buf);
  *plist = list;
  return 0;
}

// writes list through pipe, returns 0 on success.
int slist_pipeWrite(char*** plist, const char* pipecmd)
{
  int i; // loop index
  char** list = *plist;
  FILE* fp = popen(pipecmd, "w");
  if(! fp) return 1;
  for(i = 0; i < slist_count(&list); i++)
  {
    char* line = list[i]; // to see in debugger
    fprintf(fp, "%s\n", line);    
  }
  fclose(fp);
  return 0;
}

// reads lines (or args list) from a file, requires an initialized
// slist object.  Returns 0 on success.
int slist_fileRead(char*** plist, const char* filename)
{
  int intNu = 0;
  intNu = intNu;
  char** list = *plist;
  size_t n = 64000;
  char* buf = (char*)malloc(n);
  if(!buf) return 1;
  FILE* fp = fopen(filename, "r");
  if(fp)
  {
    while(!feof(fp))
    {
      *buf = 0;
      intNu = getline(&buf, &n, fp);
      if(!*buf)
        break;
      trim_newline(buf);
      slist_append(&list, buf);
    }
    fclose(fp);
  }
  free(buf);
  *plist = list;
  return 0;
}


// write lines (or args list) to a file, return 0 on success,
// list doesn't have to be a literal slist object, any null 
// terminated list not having newlines at end of lines will work.
// Optionally add a custom line_end string such as "\r\n" to each line.
int slist_fileWrite(char*** plist, const char* filename, const char* line_end)
{
  int i = 0;
  char** list = *plist;
  FILE* fp = fopen(filename, "w");
  if(!fp) return 1; // can't write, 
  // bug - If no newline, the lines don't read back as lines.
  // hopefully this will fix that withoug breaking anything else.
  if(!line_end)
    line_end = "\n"; // default
  
  while(list[i])
      fprintf(fp, "%s%s", list[i++], line_end);
  fclose(fp);
  return 0;
}


// print out a list of strings to stderr
void slist_dbg_printout(char*** plist)
{
  char** p = *plist;
  while(*p)
    fprintf(stderr, "%s\n", *p++);
}

