// tmpstr.cpp

#include <malloc.h> // realloc()
#include <stdio.h>  // vnsprintf()
#include <stdarg.h> // va_list()
#include <stdlib.h> // exit()
#include "runc.h"

static void* tmps[TMPMEM_MAX_BUFFERS] = {0};
static int tmps_index = 0;
static int max_buffers = TMPMEM_DEFAULT_BUFFERS;
static int inited = 0;


static void tmpstr_exit()
{
  if(inited)
  {
    for(int i = 0; i < TMPMEM_MAX_BUFFERS; i++)
    {
      if(tmps[i])
	free(tmps[i]);
      tmps[i] = 0;
    }
  }
}

static void init_check()
{
  if(!inited) 
    atexit(tmpstr_exit); inited++;
}

// get a temp block of memory of size bytes.
void* tmpmem(int size)
{
  init_check();
  void* buf = tmps[tmps_index];
  if(buf)
    buf = realloc(buf, size);
  else
    buf = malloc(size);
  tmps[tmps_index++] = buf;
  if(!buf) {
    fprintf(stderr, "Fatal error: Can't allocate temp buffer (tmpmem/tmpstr)\n");
    exit(1);
  }
  if(tmps_index >= max_buffers) tmps_index = 0;
  return buf;
}

  
/// Returns a tempstring from a recycling buffer up to size bytes long
char* tmpstr(int size_max, const char* fmt, ...)
{
  char* mem = (char*)tmpmem(size_max);
  *mem = 0; // null string
  if(fmt != 0)
  {
    va_list args;
    va_start(args, fmt);
    vsnprintf(mem, size_max, fmt, args);
    va_end(args);
  }
  return mem;  
}

int tmpmem_adopt(void* obj)
{
  int i; // loop index
  for(i = 0; i < max_buffers; i++) {
    if(tmps[i] == obj) {
      tmps[i] = 0;
      return 0;
    }
  } // we we get here, it's technically an error.
  return 1;
}

int tmpstr_adopt(char* obj) { return tmpmem_adopt((void*) obj);}

int tmpstr_reserve(int n)
{
  if(n > TMPMEM_MAX_BUFFERS)
    n = TMPMEM_MAX_BUFFERS;
  if(n < TMPMEM_DEFAULT_BUFFERS)
    n = TMPMEM_DEFAULT_BUFFERS;
  max_buffers = n;
  return n;
}


