// First cut at a C/C++ interpreter compiler... skipping the 'interpreter' part.  :-)

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h> // system()
#include <unistd.h> // sleep()

// make sure to use current versions while editing/modifying sources
#include "version.h"
#include "../include/runc.h"

void dbg(){} // for a non-moving breakpoint
int usage(int err);

// fill buf with args quoting as required
void flatten_args(char* buf, char** args);

const char* apppath;
const char* appname;
const char* inpath;
const char* srcname;
char execname[512];
char cppfilename[512];
void print_help();
void show(const char* what);
char private_dir[256];

// debugging aid
void slist_print(char*** plist);

int main(int argc, char** argv)
{
  dbg();
  
  int err = 0;
  char* ebuf;
  int ebufsize = 0;
  const char* compilername = "g++";
  char flags[256];
  
  // init paths and app name
  apppath = argv[0];
  appname = basename(apppath);
  sprintf(private_dir, "%s/.runc", getenv("HOME"));
  err = system(tmpstr(256, "mkdir %s 2>/dev/null", private_dir));

  if(argc < 2)
    return usage(1);
  
  if(strcmp(argv[1], "--help") == 0)
    print_help();
  
  if(strcmp(argv[1], "--show") == 0)
    show(argv[2]);
  
  // init executable name and source file name
  inpath = argv[1];
  srcname = basename(inpath);
  
  sprintf(execname, "%s/runc-%s.exec", private_dir, srcname);
  sprintf(cppfilename, "%s/runc-%s.cpp", private_dir, srcname);
  
  // init compiler flags and set default
  flags[0] = 0;
  strcat(flags, "-lrunc ");
  
  // skip first line if it starts with #! ala a shell script and copy the rest to a 
  // temp c++ source file
  char** lines = slist_new();
  
  slist_fileRead(&lines, inpath);
  // slist_dbg_printout(&lines);
  if(slist_count(&lines) && memcmp(slist_get(&lines, 0), "#!", 2) == 0)
  {
    slist_remove(&lines, 0);
    slist_insert(&lines, 0, "#line 2");
  }
  slist_fileWrite(&lines, cppfilename);
  
  // done with that list, now use the lines list for the input args
  slist_clear(&lines);
  slist_append(&lines, execname); // argv[0]
  for(int i = 2; argv[i] != 0; i++)
    slist_append(&lines, argv[i]);
  
  // get estimated total size required for commandline with possibly quoted args.
  for(int i = 0; i < slist_count(&lines); i++)
    ebufsize += strlen(slist_get(&lines, i)) + 3;
  
  ebuf = tmpstr(ebufsize, NULL);
  
  flatten_args(ebuf, lines);
  
  // try to compile and run it.
  // char* see = tmpstr(256, "%s %s %s -o %s 2> %s/runc-errlog0", compilername, cppfilename, flags, execname, private_dir);
  if(!system(tmpstr(256, "%s %s %s -o %s 2> %s/runc-errlog0", compilername, cppfilename, flags, execname, private_dir)))
  {
    remove("runc-errlog0"); // no error, no longer needed
    err = system(ebuf);
    if(err) // error during run
      fprintf(stderr, "\nRUNC: RUN ERROR\nrunc returned error code: %d\n", err);
  }
  else // error during compile
  {
    fprintf(stderr, "\nRUNC: COMPILE ERROR\n");
    sleep(2);
    
    // quick and dirty replace strings
    err = system(tmpstr(256, "cat %s/runc-errlog0 | sed 's|%s|%s|' > %s/runc-errlog", private_dir, cppfilename, srcname, private_dir));
    remove(tmpstr(256, "%s/runc-errlog0", private_dir));
    err = system(tmpstr(256, "xterm -title \"runc-errlog\" -geom 123x20 -hold -e cat %s/runc-errlog 2>/dev/null", private_dir));
    if(err)
      fprintf(stderr, "Can't find 'xterm' for error display\nSee ~/.runc/runc-errlog for error list\n");
    else
    {
      sleep(1); // give xterm time to open the file
      remove(tmpstr(256, "%s/runc-errlog", private_dir));
    }
  }
  // clean up
  remove(execname);
  remove(cppfilename);
  slist_delete(&lines);
  return err;
}

int usage(int err)
{
  fprintf(err == 0 ? stdout : stderr, "RUNC: USAGE ERROR\nType %s --help for info\n", appname);
  return err;
}

// fill buf with args quoting as required
void flatten_args(char* buf, char** args)
{
  char** arg = args;
  char* p = buf;
  int quoting;
  
  while(*arg)
  {
    quoting = 0;
    if(strchr(*arg, ' '))
    {
      *p++ = '"';
      quoting ++;
    }
    p += sprintf(p, "%s", *(arg++));
    if(quoting)
      *p++ = '"';
    *p++ = ' ';
  }
  // trim trailing?  why not...
  if(p != buf)
    p[-1] = 0;
}

void print_help()
{
  printf(
  "\n"
  "   Usage: N/A, runc is not usually called directly.\n"
  "\n"
  "   Put c code to run in a test file with '#!/usr/local/runc' on the top line\n"
  "   and make the file executable.\n"
  "\n"
  "   It will then run the C/C++ code as though it were a shell script.\n"
      "\n"
  "   To see a list of headers included with '#include <runc.h>' type\n"
  "     runc --show headers\n"
  "   To see an individual header type\n"
  "     runc --show HeaderName\n"
  "\n"
    
  );
  exit(0);
}

void show(const char* what)
{
  if(!what)
    what = "headers";
  int err = 0;
  err = err; // nu
  //  char* titlestr = tmpstr(256, "runc show: %s", what);
  if(strcmp(what, "headers") == 0)
  {
    err = system(tmpstr(256, "ls /usr/local/include/runclib > %s/runc-tmp.txt", private_dir));
    err = system(tmpstr(256, "xdg-open %s/runc-tmp.txt", private_dir));
  }
  else
    err = system(tmpstr(256, "xdg-open file:///usr/local/include/runclib/%s 2>/dev/null", what));
  
  if(err)
    printf("You may need to install xdg-open or set the default application to open C headers\n");
  
  exit(0);
}

