// tmpstr.h

#ifndef LQ_tmpstr_h
#define LQ_tmpstr_h

/** \file tmpstr.h
 * tmpstr and tmpmem return temporary recycling blocks of memory for 
 * immediate use.  Tmpstr also allows string formatting commands similar 
 * to sprintf() and there is no penalty for overestimating the size of the 
 * final string though underestimating can cause crashes.  Up to 1024 strings 
 * and blocks of strings and/or memory can exist at once before the oldest 
 * block gets recycled.  \sa tmpstr_reserve()
 */

// The number of concurrent blocks of mem or strings that can be used 
// before they start recycling.  Currently up to 1024 concurrent strings 
// may be used before recycling, default is 32 concurrent.
#define TMPMEM_MAX_BUFFERS 1024
#define TMPMEM_DEFAULT_BUFFERS 32

/// Returns a memory temp object from a recycling buffer of size bytes
void* tmpmem(int size);

/// Returns a tempstring from a recycling buffer up to size bytes long
char* tmpstr(int size_max, const char* fmt, ...);

/** Takes over ownership of an object or string.  After adoption the user
 is responsible for freeing the object when no longer needed.  Returns 
0 on success, else unable to adopt.*/
int tmpmem_adopt(void* p);

/// an alias for tmpmem_adopt() for use with strings.
int tmpstr_adopt(char* p);

/** reserve n strings (modulus) before recycling.  Returns number actually
 reserved.  Limit is set by the TMPMEM_MAX_BUFFERS constant*/
int tmpstr_reserve(int n);

#endif // LQ_tmpstr_h

